package com.fjhx.iot.service.tda.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.iot.entity.tda.dto.TdaDeviceDto;
import com.fjhx.iot.entity.tda.dto.TdaDeviceSelectDto;
import com.fjhx.iot.entity.tda.po.TdaApplication;
import com.fjhx.iot.entity.tda.po.TdaDevice;
import com.fjhx.iot.entity.tda.po.TdaProduct;
import com.fjhx.iot.entity.tda.vo.TdaDeviceVo;
import com.fjhx.iot.mapper.tda.TdaDeviceMapper;
import com.fjhx.iot.service.tda.TdaApplicationService;
import com.fjhx.iot.service.tda.TdaDeviceService;
import com.fjhx.iot.service.tda.TdaProductService;
import com.fjhx.iot.utils.IoTDAUtil;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;


/**
 * <p>
 * 设备管理 服务实现类
 * </p>
 *
 * @author zlj
 * @since 2023-03-21
 */
@Service
public class TdaDeviceServiceImpl extends ServiceImpl<TdaDeviceMapper, TdaDevice> implements TdaDeviceService {

    @Lazy
    @Autowired
    private TdaProductService tdaProductService;

    @Lazy
    @Autowired
    private TdaApplicationService tdaApplicationService;


    @Override
    public Page<TdaDeviceVo> getPage(TdaDeviceSelectDto dto) {
        IWrapper<TdaDevice> wrapper = getWrapper();
        wrapper.orderByDesc("td", TdaDevice::getId);
        wrapper.keyword(dto, new SqlField("td.device_name"));
        Page<TdaDeviceVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public TdaDeviceVo detail(Long id) {
        TdaDevice TdaDevice = this.getById(id);
        TdaDeviceVo result = BeanUtil.toBean(TdaDevice, TdaDeviceVo.class);
        return result;
    }

    @Override
    public void add(TdaDeviceDto tdaDeviceDto) {
        String nodeId = tdaDeviceDto.getNodeId();
        String deviceName = tdaDeviceDto.getDeviceName();
        Long tdaProductId = tdaDeviceDto.getTdaProductId();

        // 查询资源所在空间和产品id
        TdaProduct tdaProduct = tdaProductService.getById(tdaProductId);
        String productId = tdaProduct.getProductId();

        TdaApplication tdaApplication = tdaApplicationService.getById(tdaProduct.getTdaApplicationId());

        // 华为ioDta添加设备
        IoTDAUtil.DeviceKey deviceKey = IoTDAUtil.addDevice(
                tdaApplication.getTdaConfigId(), nodeId, deviceName, tdaProduct.getAppId(), productId);

        tdaDeviceDto.setProductId(productId);
        tdaDeviceDto.setDeviceId(deviceKey.getDeviceId());
        tdaDeviceDto.setSecret(deviceKey.getSecret());

        this.save(tdaDeviceDto);
    }

    @Override
    public void delete(Long id) {

        TdaDevice tdaDevice = getById(id);
        TdaProduct tdaProduct = tdaProductService.getById(tdaDevice.getTdaProductId());
        TdaApplication tdaApplication = tdaApplicationService.getById(tdaProduct.getTdaApplicationId());

        IoTDAUtil.deleteDevice(tdaApplication.getTdaConfigId(), tdaDevice.getDeviceId());

        this.removeById(id);
    }

}
